class HRuleBreakpoint(gdb.Breakpoint):
	def stop(self):
		frame = gdb.selected_frame()
		block = frame.block()

		for p in block:
			top_level_parse.parser_objs[int(p.value(frame))] = Parser(p.name, int(p.value(frame)))

class BreakpointManager:
	def __init__(self, h_rule_functions):
		self.hammer_retq_breakpoints = []
		self.h_rule_breakpoints = []
		self.H_RULE_FUNCTIONS = h_rule_functions

		self.h_do_parse = None
		self.h_packrat_parse = None
		self.perform_lowlevel_parse = None
		self.h_arena_malloc_raw = None

		self.parse_action = None
		self.parse_choice = None
		self.parse_sequence = None
		self.parse_difference = None
		self.parse_many = None
		self.parse_and = None
		self.parse_attr_bool = None
		self.parse_bind = None
		self.parse_bits = None
		self.parse_butnot = None
		self.parse_charset = None
		self.parse_ch = None
		self.parse_end = None
		self.parse_endianness = None
		self.parse_epsilon = None
		self.parse_ignore = None
		self.parse_ignoreseq = None
		self.parse_indirect = None
		self.parse_int_range = None
		self.parse_not = None
		self.parse_nothing = None
		self.parse_optional = None
		self.parse_permutation = None
		self.parse_skip = None
		self.parse_seek = None
		self.parse_tell = None
		self.parse_token = None
		self.parse_unimplemented = None
		self.parse_put = None
		self.parse_get = None
		self.parse_whitespace = None
		self.parse_xor = None

	def set_h_rule_breakpoints(self):
		for func in self.H_RULE_FUNCTIONS:
			func_retq = self.locate_retq(func[0], func[1])
			self.h_rule_breakpoints.append(HRuleBreakpoint("*" + hex(func_retq)))

	def set_hammer_retq_breakpoints(self):
		plp_retq = self.locate_perform_lowlevel_parse_retq()
		perform_lowlevel_parse_ret = PerformLowLevelParseRetBreakpoint("*" + hex(plp_retq))
		self.hammer_retq_breakpoints.append(perform_lowlevel_parse_ret)
		hpp_retq = locate_h_packrat_parse_retq()
		h_packrat_parse_ret = HPackratParseRetBreakpoint("*" + hex(hpp_retq))
		self.hammer_retq_breakpoints.append(h_packrat_parse_ret)
		return self.hammer_retq_breakpoints

	def del_hammer_retq_breakpoints(self):
		for bp in self.hammer_retq_breakpoints:
			bp.delete()

	# Helper functions for finding the return instructions of particular functions of interest

	def locate_retqs(self, symbol, length):
		arch = gdb.selected_frame().architecture()
		sym = gdb.lookup_symbol(symbol)[0]
		sym_address = int(sym.value().address)
		instructions = arch.disassemble(sym_address, sym_address+length)
		results = [ ins["addr"] for ins in instructions if ins["asm"].startswith("ret") ]
		return results

	def locate_retq(self, symbol, length):
		results = self.locate_retqs(symbol, length)
		return results[0]

	# TODO: these can probably be removed

	def locate_h_packrat_parse_retq(self):
		arch = gdb.selected_frame().architecture()
		h_p_p_sym = gdb.lookup_symbol("h_packrat_parse")[0]
		h_p_p_address = int(h_p_p_sym.value().address)
		# Same as with perform_lowlevel_parse, +450 is arbitrary
		instructions = arch.disassemble(h_p_p_address, h_p_p_address+450)
		results = [ ins["addr"] for ins in instructions if ins["asm"].startswith("ret") ]
		return results[0]

	def locate_init_parser_retq(self):
		arch = gdb.selected_frame().architecture()
		i_p_sym = gdb.lookup_symbol("init_parser")[0]
		i_p_address = int(i_p_sym.value().address)
		# Same as with perform_lowlevel_parse, +16000 is arbitrary
		instructions = arch.disassemble(i_p_address, i_p_address+16000)
		results = [ ins["addr"] for ins in instructions if ins["asm"].startswith("ret") ]
		return results[0]

	def locate_perform_lowlevel_parse_retq(self):
		arch = gdb.selected_frame().architecture()
		p_l_p_sym = gdb.lookup_symbol("perform_lowlevel_parse")[0]
		p_l_p_address = int(p_l_p_sym.value().address)
		# The choice of disassembling only 400 instructions from the start is arbitrary. (This function is 310 bytes long on this particular machine.) There is probably a way to find out where a function ends.
		instructions = arch.disassemble(p_l_p_address, p_l_p_address+400)
		results = [ ins["addr"] for ins in instructions if ins["asm"].startswith("ret") ]
		return results[0]
